/*
 * Copyright (c) 2013-2021 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Layouts 1.12
import Shotcut.Controls 1.0 as Shotcut

ToolBar {
    property alias scrub: scrubButton.checked
    property color checkedColor: Qt.rgba(activePalette.highlight.r, activePalette.highlight.g, activePalette.highlight.b, 0.4)
    property alias scaleSlider: scaleSlider
    property alias mark1: m1.text
    property alias mark2: m2.text
    property alias mark3: m3.text
    property alias mark4: m4.text
    property alias mark5: m5.text
    property alias colorMark: colorMark
    property alias deplacement: deplacement
    property int selectedmark: 0
    property var reforigine : ['0', m1.text, m2.text, m3.text, m4.text, m5.text]
    property var choixcolor : [false, false, false, false, false, false]
    property int j: 0

    SystemPalette { id: activePalette }

    id: toolbar
    width: 200
    height: settings.smallIcons ? 25 : 33
    
    function isMarker() {
        if (timeline.position == m1.text || timeline.position == m2.text || timeline.position == m3.text || timeline.position == m4.text || timeline.position == m5.text) return true
        else return false
    }

    function isMarkerFull() {
        if (m1.text !== '0' && m2.text !== '0' && m3.text !== '0' && m4.text !== '0' && m5.text !== '0') return true
    }

    function choixRef(refmark) {
        selectedmark = refmark
        origine.text = reforigine[refmark]
        for (var i = 0; i < 6; i ++) { 
            choixcolor[i] = (i === refmark)? true : false
        }
        choixcolor0.checked = choixcolor[0]
        choixcolor1.checked = choixcolor[1]
        choixcolor2.checked = choixcolor[2]
        choixcolor3.checked = choixcolor[3]
        choixcolor4.checked = choixcolor[4]
        choixcolor5.checked = choixcolor[5]
    }
    
    function setMarker() {
        j = 0
        if      (m1.text == '0') {m1.text = timeline.position; j = 1}
        else if (m2.text == '0') {m2.text = timeline.position; j = 2}
        else if (m3.text == '0') {m3.text = timeline.position; j = 3}
        else if (m4.text == '0') {m4.text = timeline.position; j = 4}
        else if (m5.text == '0') {m5.text = timeline.position; j = 5}
    }
    
    function deleteMarker() {
        if (timeline.position == m1.text) {
            if (timeline.position == origine.text) {selectedmark = 0 ; origine.text = '0'}
            m1.text = '0'
            choixcolor1.checked = false
        }
        if (timeline.position == m2.text) {
            if (timeline.position == origine.text) {selectedmark = 0 ; origine.text = '0'}
            m2.text = '0'
            choixcolor2.checked = false
        }
        if (timeline.position == m3.text) {
            if (timeline.position == origine.text) {selectedmark = 0 ; origine.text = '0'}
            m3.text = '0'
            choixcolor3.checked = false
        }
        if (timeline.position == m4.text) {
            if (timeline.position == origine.text) {selectedmark = 0 ; origine.text = '0'}
            m4.text = '0'
            choixcolor4.checked = false
        }
        if (timeline.position == m5.text) {
            if (timeline.position == origine.text) {selectedmark = 0 ; origine.text = '0'}
            m5.text = '0'
            choixcolor5.checked = false
        }
    }
    
    function previous() {
        var act = parseInt(timeline.position)
        var pre = 0
        for (var i = 1; i <= 6; i ++) {
            if (parseInt(reforigine[i]) > pre && parseInt(reforigine[i]) < act) pre = parseInt(reforigine[i])
        }
        timeline.position = pre
    }
    
    function next() {
        var act = parseInt(timeline.position)
        var nex = act
        for (var i = 1; i <= 6; i ++) {
            if (parseInt(reforigine[i]) > nex) nex = parseInt(reforigine[i])
        }
        for (var i = 1; i <= 6; i ++) {
            if (parseInt(reforigine[i]) < nex && parseInt(reforigine[i]) > act) nex = parseInt(reforigine[i])
        }
        timeline.position = nex
    }
    
    RowLayout {
        y: 2

        Shotcut.ToolButton {
            id: menuButton
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: menuAction
            Shotcut.HoverTip { text: qsTr('Display a menu of additional actions') }
            focusPolicy: Qt.NoFocus
        }
        Button { // separator
            enabled: false
            implicitWidth: 2
            implicitHeight: toolbar.height / 2
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: cutAction
            Shotcut.HoverTip { text: qsTr('Cut - Copy the current clip to the Source\nplayer and ripple delete it') }
            focusPolicy: Qt.NoFocus
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: copyAction
            Shotcut.HoverTip { text: qsTr('Copy - Copy the current clip to the Source player (C)') }
            focusPolicy: Qt.NoFocus
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: insertAction
            Shotcut.HoverTip { text: qsTr('Paste - Insert clip into the current track\nshifting following clips to the right (V)') }
            focusPolicy: Qt.NoFocus
        }
        Button { // separator
            enabled: false
            implicitWidth: 2
            implicitHeight: toolbar.height / 2
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: appendAction
            Shotcut.HoverTip { text: qsTr('Append to the current track (A)') }
            focusPolicy: Qt.NoFocus
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: deleteAction
            Shotcut.HoverTip { text: qsTr('Ripple Delete - Remove current clip\nshifting following clips to the left (X)') }
            focusPolicy: Qt.NoFocus
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: liftAction
            Shotcut.HoverTip { text: qsTr('Lift - Remove current clip without\naffecting position of other clips (Z)') }
            focusPolicy: Qt.NoFocus
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: overwriteAction
            Shotcut.HoverTip { text: qsTr('Overwrite clip onto the current track (B)') }
            focusPolicy: Qt.NoFocus
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: splitAction
            Shotcut.HoverTip { text: qsTr('Split At Playhead (S)') }
            focusPolicy: Qt.NoFocus
        }
        Button { // separator
            enabled: false
            implicitWidth: 2
            implicitHeight: toolbar.height / 2
        }
        Shotcut.ToolButton {
            id: snapButton
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            checked: settings.timelineSnap
            icon.name: 'snap'
            icon.source: 'qrc:///icons/oxygen/32x32/actions/snap.png'
            focusPolicy: Qt.NoFocus
            Shotcut.HoverTip { text: qsTr('Toggle snapping') }
            onClicked: settings.timelineSnap = !settings.timelineSnap
        }
        Shotcut.ToolButton {
            id: scrubButton
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            checked: settings.timelineDragScrub
            icon.name: 'scrub_drag'
            icon.source: 'qrc:///icons/oxygen/32x32/actions/scrub_drag.png'
            focusPolicy: Qt.NoFocus
            Shotcut.HoverTip { text: qsTr('Scrub while dragging') }
            onClicked: settings.timelineDragScrub = !settings.timelineDragScrub
        }
        Shotcut.ToolButton {
            id: rippleButton
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            checked: settings.timelineRipple
            icon.name: 'target'
            icon.source: 'qrc:///icons/oxygen/32x32/actions/target.png'
            focusPolicy: Qt.NoFocus
            Shotcut.HoverTip { text: qsTr('Ripple trim and drop') }
            onClicked: settings.timelineRipple = !settings.timelineRipple
        }
        Shotcut.ToolButton {
            id: rippleAllButton
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            checked: settings.timelineRippleAllTracks
            icon.name: 'ripple-all'
            icon.source: 'qrc:///icons/oxygen/32x32/actions/ripple-all.png'
            focusPolicy: Qt.NoFocus
            Shotcut.HoverTip { text: qsTr('Ripple edits across all tracks') }
            onClicked: settings.timelineRippleAllTracks = !settings.timelineRippleAllTracks
        }
// **************** Début modif déplacement playhead *********
        Button { // separator
            enabled: false
            visible: deplacement.visible
            implicitWidth: 2
            implicitHeight: toolbar.height / 2
        }
        
        Rectangle {
            id: deplacement
            visible: false
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight * 2 + moveValue.width - 4
            color: '#00000000'
            border.color: 'gray'
            border.width: 1
            radius: 3
            Shotcut.ToolButton {
                implicitHeight: parent.height - 2
                implicitWidth: implicitHeight
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: parent.left
                anchors.leftMargin: 1
                icon.name: 'media-seek-backward'
                icon.source: 'qrc:///icons/oxygen/32x32/actions/media-seek-backward.png'
                //rotation: 180
                Shotcut.HoverTip { text: qsTr('Move the Playhead backward ' + parseInt(moveValue.text) + ' frames.') }
                onClicked: timeline.position = timeline.position - parseInt(moveValue.text)
            }
            TextField {
                id: moveValue
                selectByMouse: true
                horizontalAlignment: TextInput.AlignHCenter
        	    text: '10'
        	    font.family: 'Verdana'
                font.pointSize: (application.OS === 'OS X')? 11 : 8
                implicitWidth: 34
                implicitHeight: parent.height - 2
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                Shotcut.HoverTip { text: qsTr('Choose the number of frames the playhead will skip') }
        	    Keys.onReturnPressed: focus = false
        	    Keys.onEnterPressed: focus = false
            }
            Shotcut.ToolButton {
                implicitHeight: parent.height - 2
                implicitWidth: implicitHeight
                anchors.verticalCenter: parent.verticalCenter
                anchors.right: parent.right
                anchors.rightMargin: 1
                icon.name: 'media-seek-forward'
                icon.source: 'qrc:///icons/oxygen/32x32/actions/media-seek-forward.png'
                Shotcut.HoverTip { text: qsTr('Move the Playhead forward ' + parseInt(moveValue.text) + ' frames.') }
                onClicked: timeline.position = timeline.position + parseInt(moveValue.text)
            }
        }
// ********** fin modif déplacement playhead ************************
        Button { // separator
            visible: colorMark.visible
            enabled: false
            implicitWidth: 2
            implicitHeight: toolbar.height / 2
        }
// ********** Début modif Marqueurs ************************
        TextField {
            id: origine
            text: "0"
            visible: false
        }
        TextField {
            id: m1
            text: "0"
            visible: false
        }
        TextField {
            id: m2
            text: "0"
            visible: false
        }
        TextField {
            id: m3
            text: "0"
            visible: false
        }
        TextField {
            id: m4
            text: "0"
            visible: false
        }
        TextField {
            id: m5
            text: "0"
            visible: false
        }
        
        Rectangle {
            id: colorMark
            implicitHeight: toolbar.height - 3
            implicitWidth: (icons_long.checked)? implicitHeight * 4 - 4 : implicitHeight
            visible: false
            color: '#00000000'
            border.color: 'gray'
            border.width: 1
            radius: 3
            Shotcut.ToolButton {
                id: precedent
                visible: icons_long.checked
                implicitHeight: parent.height - 2
                implicitWidth: implicitHeight
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: parent.left
                anchors.leftMargin: 1
                icon.name: 'media-skip-backward'
                icon.source: 'qrc:///icons/oxygen/32x32/actions/media-skip-backward.png'
                Shotcut.HoverTip { text: qsTr('Move to previous marker') }
                onClicked: previous()
            }
            Shotcut.ToolButton { // Menu marqueur (*)
                id: buttonmenu
                implicitHeight: parent.height - 2
                implicitWidth: implicitHeight
                visible: true
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: (icons_long.checked)? precedent.right : parent.left
                anchors.leftMargin: 1
                icon.name: 'bookmarks'
                icon.source: 'qrc:///icons/oxygen/32x32/status/bookmarks.png'
                Shotcut.HoverTip { text: qsTr('Display the marker menu') }
                onClicked: {
                    menuMark.popup()
                }
            }
            Shotcut.ToolButton { // Bouton +
                id: buttonAdd
                visible: icons_long.checked && !isMarker()
                implicitHeight: parent.height - 2
                implicitWidth: implicitHeight
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: buttonmenu.right
                anchors.leftMargin: 1
                enabled: !isMarkerFull() && timeline.position >= 1
                icon.name: 'list-add'
                icon.source: 'qrc:///icons/oxygen/32x32/actions/list-add.png'
                Shotcut.HoverTip { text: qsTr('Add a marker here') }
                onClicked: setMarker()
            }
            Shotcut.ToolButton { // Bouton -
                visible: icons_long.checked && !buttonAdd.visible
                implicitHeight: parent.height - 2
                implicitWidth: implicitHeight
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: buttonmenu.right
                anchors.leftMargin: 1
                enabled: timeline.position >= 1
                icon.name: 'list-remove'
                icon.source: 'qrc:///icons/oxygen/32x32/actions/list-remove.png'
                Shotcut.HoverTip { text: qsTr('Delete this marker') }
                onClicked: deleteMarker()
            }
            Shotcut.ToolButton {
                id: suivant
                visible: icons_long.checked
                implicitHeight: parent.height - 2
                implicitWidth: implicitHeight
                anchors.verticalCenter: parent.verticalCenter
                anchors.right: parent.right
                anchors.rightMargin: 1
                icon.name: 'media-skip-forward'
                icon.source: 'qrc:///icons/oxygen/32x32/actions/media-skip-forward.png'
                Shotcut.HoverTip { text: qsTr('Move to next marker') }
                onClicked: next()
            }
        }
        Menu { // Menu de l'icône Marqueurs
            id: menuMark
            MenuItem { 
                text: isMarkerFull()? qsTr('All markers are in use !') : qsTr('Add a marker here')
                implicitHeight: 22
                enabled: !isMarkerFull() && !isMarker()
                onTriggered: setMarker()
            }
            MenuItem { 
                text: qsTr('Delete this marker')
                implicitHeight: 22
                enabled: isMarker() && timeline.position >= 1
                onTriggered: deleteMarker()
            }
            MenuItem { 
                text: qsTr('Delete all markers')
                implicitHeight: 22
                onTriggered: {
                    selectedmark = 0
                    origine.text = m1.text = m2.text = m3.text = m4.text = m5.text = '0'
                    choixcolor1.checked = choixcolor2.checked = choixcolor3.checked = choixcolor4.checked = choixcolor5.checked = false
                }
            }
            MenuSeparator {
                padding: 0
                topPadding: 3
                bottomPadding: 3
                contentItem: Rectangle {
                    implicitWidth: 250
                    implicitHeight: 1
                    color: 'gray'
                }
            }
            Menu {
                title: qsTr('Move Playhead to Marker')
                //
                MenuItem {
                    text: qsTr('Previous Marker')
                    implicitHeight: icons_long.checked? 0 : 22
                    onTriggered: previous()
                }
                MenuItem {
                    text: qsTr('Next Marker')
                    implicitHeight: icons_long.checked? 0 : 22
                    onTriggered: next()
                }
                MenuSeparator {
                    padding: 0
                    topPadding: 3
                    bottomPadding: 3
                    contentItem: Rectangle {
                        implicitWidth: 250
                        implicitHeight: 1
                        color: 'gray'
                    }
                }
                MenuItem {
                    text: qsTr('Start (0:00:00:00)')
                    icon.source: 'icons/black.png'
                    enabled: timeline.position != 0
                    implicitHeight: 22
                    onTriggered: timeline.position = 0
                }
                MenuItem {
                    text: qsTr('Blue') + ' (' + application.timecode(m1.text).substr(1, 10) + ')'
                    icon.source: 'icons/bleu.png'
                    implicitHeight: (m1.text !== '0')? 22 : 0
                    enabled: timeline.position != m1.text
                    onTriggered: timeline.position = m1.text
                }
                MenuItem {
                    text: qsTr('Yellow') + ' (' + application.timecode(m2.text).substr(1, 10) + ')'
                    icon.source: 'icons/yellow.png'
                    implicitHeight: (m2.text !== '0')? 22 : 0
                    enabled: timeline.position != m2.text
                    onTriggered: timeline.position = m2.text
                }
                MenuItem {
                    text: qsTr('Magenta') + ' (' + application.timecode(m3.text).substr(1, 10) + ')'
                    icon.source: 'icons/magenta.png'
                    implicitHeight: (m3.text !== '0')? 22 : 0
                    enabled: timeline.position != m3.text
                    onTriggered: timeline.position = m3.text
                }
                MenuItem {
                    text: qsTr('Orange') + ' (' + application.timecode(m4.text).substr(1, 10) + ')'
                    icon.source: 'icons/orange.png'
                    implicitHeight: (m4.text !== '0')? 22 : 0
                    enabled: timeline.position != m4.text
                    onTriggered: timeline.position = m4.text
                }
                MenuItem {
                    text: qsTr('Cyan') + ' (' + application.timecode(m5.text).substr(1, 10) + ')'
                    icon.source: 'icons/cyan.png'
                    implicitHeight: (m5.text !== '0')? 22 : 0
                    enabled: timeline.position != m5.text
                    onTriggered: timeline.position = m5.text
                }
            }
            MenuSeparator {
                padding: 0
                topPadding: 3
                bottomPadding: 3
                contentItem: Rectangle {
                    implicitWidth: 250
                    implicitHeight: 1
                    color: 'gray'
                }
            }
            MenuItem { 
                id: icons_long
                text: qsTr('Show extended Toolbox')
                implicitHeight: 22
                checkable: true
                checked: false
            }
            MenuItem { 
                id: aff_compteur
                text: qsTr('Display the timer')
                implicitHeight: 22
                checkable: true
                checked: false
            }
            MenuItem {
                text: qsTr('Cancel')
                implicitHeight: 22
            }
        }
                        
        Rectangle { // Compteur
            id: timer
            visible: aff_compteur.checked && colorMark.visible
            implicitWidth: 80
            implicitHeight: toolbar.height - 3
            color: '#00000000' //'#FF1E1E1E'
            border.color: {
                (selectedmark == 1)? '#FF0090FF' : 
                (selectedmark == 2)? '#FFFFFF00' : 
                (selectedmark == 3)? '#FFFF00FF' : 
                (selectedmark == 4)? '#FFFF6600' : 
                (selectedmark == 5)? '#FF66FFFF' :
                '#FF808080'
            }
            border.width: 1
            radius: 3
            TextField {
                text: (timeline.position - origine.text >= 0)? " " + application.timecode(timeline.position - origine.text).substr(1, 10) : "-" + application.timecode(origine.text - timeline.position).substr(1, 10)
                font.family: 'Verdana'
                font.pointSize: (application.OS === 'OS X')? 11 : 8
                horizontalAlignment: TextInput.AlignRight
                implicitWidth: parent.width - 2
                implicitHeight: parent.height - 2
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                readOnly: true
             }   
            MouseArea {
                anchors.fill: parent
                acceptedButtons: Qt.LeftButton
                Shotcut.HoverTip { text: qsTr('Clic to choose a reference') }
                onClicked: {
                    menuTimer.popup()
                }
            }
        }
        Menu { // Menu de la zone compteur
            id: menuTimer
            implicitWidth: 320
            MenuItem { 
                text: qsTr('Choose actual playhead position as reference')
                implicitHeight: 22
                enabled: !isMarkerFull() && !isMarker() && timeline.position >= 1
                onTriggered: {
                    setMarker()
                    choixRef(j)
                }
            }
            MenuSeparator {
                padding: 0
                topPadding: 3
                bottomPadding: 3
                contentItem: Rectangle {
                    implicitWidth: 250
                    implicitHeight: 1
                    color: 'gray'
                }
            }
            MenuItem { 
                id: choixcolor0
                enabled: selectedmark != 0
                text: (selectedmark == 0)? qsTr('Start is the actual reference') : qsTr('Choose Start (0:00:00:00)')
                icon.source: 'icons/black.png'
                implicitHeight: 22
                checkable: true
                checked: origine.text == '0'
                onTriggered: {
                    selectedmark = 0
                    origine.text = '0'
                    checked = true
                    choixcolor1.checked = choixcolor2.checked = choixcolor3.checked = choixcolor4.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: choixcolor1
                visible: m1.text !== '0'
                implicitHeight: (m1.text !== '0')? 22 : 0
                checkable: true
                checked: origine.text == m1.text
                enabled: selectedmark != 1
                text: (selectedmark == 1)? qsTr('Blue is the actual reference') : qsTr('Choose Blue as reference') + ' (' + application.timecode(m1.text).substr(1, 10) + ')'
                icon.source: 'icons/bleu.png'
                onTriggered: {
                    selectedmark = 1
                    origine.text = m1.text
                    checked = true
                    choixcolor0.checked = choixcolor2.checked = choixcolor3.checked = choixcolor4.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: choixcolor2
                visible: m2.text !== '0'
                implicitHeight: (m2.text !== '0')? 22 : 0
                checkable: true
                checked: origine.text == m2.text
                enabled: selectedmark != 2
                text: (selectedmark == 2)? qsTr('Yellow is the actual reference') : qsTr('Choose Yellow as reference') + ' (' + application.timecode(m2.text).substr(1, 10) + ')'
                icon.source: 'icons/yellow.png'
                onTriggered: {
                    selectedmark = 2
                    origine.text = m2.text
                    checked = true
                    choixcolor0.checked = choixcolor1.checked = choixcolor3.checked = choixcolor4.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: choixcolor3
                visible: m3.text !== '0'
                implicitHeight: (m3.text !== '0')? 22 : 0
                checkable: true
                checked: origine.text == m3.text
                enabled: selectedmark != 3
                text: (selectedmark == 3)? qsTr('Magenta is the actual reference') : qsTr('Choose Magenta as reference') + ' (' + application.timecode(m3.text).substr(1, 10) + ')'
                icon.source: 'icons/magenta.png'
                onTriggered: {
                    selectedmark = 3
                    origine.text = m3.text
                    checked = true
                    choixcolor0.checked = choixcolor1.checked = choixcolor2.checked = choixcolor4.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: choixcolor4
                visible: m4.text !== '0'
                implicitHeight: (m4.text !== '0')? 22 : 0
                checkable: true
                checked: origine.text == m4.text
                enabled: selectedmark != 4
                text: (selectedmark == 4)? qsTr('Orange is the actual reference') : qsTr('Choose Orange as reference') + ' (' + application.timecode(m4.text).substr(1, 10) + ')'
                icon.source: 'icons/orange.png'
                onTriggered: {
                    selectedmark = 4
                    origine.text = m4.text
                    checked = true
                    choixcolor0.checked = choixcolor1.checked = choixcolor2.checked = choixcolor3.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: choixcolor5
                visible: m5.text !== '0'
                implicitHeight: (m5.text !== '0')? 22 : 0
                checkable: true
                checked: origine.text == m5.text
                enabled: selectedmark != 5
                text: (selectedmark == 5)? qsTr('Cyan is the actual reference') : qsTr('Choose Cyan as reference') + ' (' + application.timecode(m5.text).substr(1, 10) + ')'
                icon.source: 'icons/cyan.png'
                onTriggered: {
                    selectedmark = 5
                    origine.text = m5.text
                    checked = true
                    choixcolor0.checked = choixcolor1.checked = choixcolor2.checked = choixcolor3.checked = choixcolor4.checked = false
                }
            }
            MenuItem {
                text: qsTr('Cancel')
                implicitHeight: 22
            }
        }
// ********** fin modif Marqueurs ************************
        Button { // separator
            enabled: false
            implicitWidth: 2
            implicitHeight: toolbar.height / 2
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: zoomOutAction
            Shotcut.HoverTip { text: qsTr("Zoom timeline out (-)") }
            focusPolicy: Qt.NoFocus
        }
        ZoomSlider {
            id: scaleSlider
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: zoomInAction
            Shotcut.HoverTip { text: qsTr("Zoom timeline in (+)") }
            focusPolicy: Qt.NoFocus
        }
        Shotcut.ToolButton {
            implicitHeight: toolbar.height - 3
            implicitWidth: implicitHeight
            action: zoomFitAction
            Shotcut.HoverTip { text: qsTr('Zoom timeline to fit (0)') }
            focusPolicy: Qt.NoFocus
        }
    }

    Action {
        id: menuAction
        icon.name: 'show-menu'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/show-menu.png'
        onTriggered: menu.popup()
    }

    Action {
        id: cutAction
        icon.name: 'edit-cut'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/edit-cut.png'
        enabled: timeline.selection.length
        onTriggered: timeline.removeSelection(true)
    }

    Action {
        id: copyAction
        icon.name: 'edit-copy'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/edit-copy.png'
        enabled: timeline.selection.length
        onTriggered: timeline.copyClip(timeline.selection[0].y, timeline.selection[0].x)
    }

    Action {
        id: insertAction
        icon.name: 'edit-paste'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/edit-paste.png'
        onTriggered: timeline.insert(currentTrack)
    }

    Action {
        id: appendAction
        icon.name: 'list-add'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/list-add.png'
        onTriggered: timeline.append(currentTrack)
    }

    Action {
        id: deleteAction
        icon.name: 'list-remove'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/list-remove.png'
        onTriggered: timeline.removeSelection()
   }

    Action {
        id: liftAction
        icon.name: 'lift'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/lift.png'
        onTriggered: timeline.liftSelection()
    }

    Action {
        id: overwriteAction
        icon.name: 'overwrite'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/overwrite.png'
        onTriggered: timeline.overwrite(currentTrack)
    }

    Action {
        id: splitAction
        icon.name: 'slice'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/slice.png'
        onTriggered: timeline.splitClip(currentTrack)
    }

    Action {
        id: zoomOutAction
        icon.name: 'zoom-out'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/zoom-out.png'
        onTriggered: root.zoomOut()
    }

    Action {
        id: zoomInAction
        icon.name: 'zoom-in'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/zoom-in.png'
        onTriggered: root.zoomIn()
    }

    Action {
        id: zoomFitAction
        icon.name: 'zoom-fit-best'
        icon.source: 'qrc:///icons/oxygen/32x32/actions/zoom-fit-best.png'
        onTriggered: root.zoomToFit()
    }
}
